@@ -2,7 +2,7 @@ package Karas;
 use strict;
 use warnings;
 use 5.010001;
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 use Carp ();
 use Class::Accessor::Lite 0.05 (
     rw => [qw/query_builder default_row_class owner_pid connection_manager row_class_map/],
@@ -12,12 +12,13 @@ use Module::Load ();
 use String::CamelCase ();
 use Data::Page::NoTotalEntries;
 use Scalar::Util ();
-use DBIx::Handler;
 use Class::Trigger qw(
     BEFORE_INSERT
 
     BEFORE_BULK_INSERT
 
+    BEFORE_INSERT_ON_DUPLICATE
+
     BEFORE_UPDATE_ROW
     AFTER_UPDATE_ROW
     BEFORE_UPDATE_DIRECT
@@ -30,6 +31,7 @@ use Class::Trigger qw(
 );
 
 use DBIx::TransactionManager;
+use DBIx::Handler;
 
 use Karas::Row;
 use Karas::QueryBuilder;
@@ -48,12 +50,11 @@ sub new {
     $args{connect_info}->[3]->{ShowErrorStatement} //= 1;
     $args{connect_info}->[3]->{AutoInactiveDestroy} //= 1;
 
-    $args{row_class_map} = $class->load_row_class_map();
-    $args{default_row_class} ||= 'Karas::Row';
     $args{connection_manager} = DBIx::Handler->new(
         @{$args{connect_info}}
     );
     my $self = bless {
+        row_class_map => {},
         %args
     }, $class;
     $self->{query_builder} ||= Karas::QueryBuilder->new(driver => $self->_driver_name);
@@ -71,6 +72,9 @@ sub _driver_name {
 # -------------------------------------------------------------------------
 sub load_plugin {
     my ($class, $name, $args) = @_;
+    Carp::croak("Do not use this plugin to instance") if ref $class;
+    Carp::croak("Do not load this plugin to Karas itself. Please make your own child class from Karas.") if $class eq 'Karas';
+
     $name = ($name =~ s/^\+//) ? $name : "Karas::Plugin::$name";
     Module::Load::load($name);
     my $plugin = $name->new($args || +{});
@@ -94,47 +98,36 @@ sub disconnect {
     $self->connection_manager->disconnect();
 }
 
-sub reconnect {
-    my $self = shift;
-    $self->connection_manager->reconnect(@_);
-}
-
 # ------------------------------------------------------------------------- 
 # schema
 #
 # -------------------------------------------------------------------------
 
-our %_LOAD_ROW_CLASS_MAP_CACHE;
-sub load_row_class_map {
-    my $class = shift;
-    return +{} if $class eq __PACKAGE__;
-
-    $_LOAD_ROW_CLASS_MAP_CACHE{$class} ||= do {
-        my %ret;
-        for my $klass (Module::Find::useall "${class}::Row") {
-            my $table_name = do {
-                if ($klass->can('table_name')) {
-                    $klass->table_name;
-                } else {
-                    my $name = $klass;
-                    $name =~ s!^${class}::Row::!!;
-                    String::CamelCase::decamelize($name);
-                }
-            };
-            $ret{$table_name} = $klass;
+sub load_schema_from_db {
+    my ($self, %args) = @_;
+    require Karas::Loader;
+    my $class = ref($self) || $self;
+    $args{namespace} //= do {
+        if ($class eq 'Karas') {
+            state $i=0;
+            $class . '::Anon' . $i++;
+        } else {
+            $class;
         }
-        \%ret;
     };
-}
-
-sub clear_row_class_map_cache {
-    %_LOAD_ROW_CLASS_MAP_CACHE = ();
+    $self->{row_class_map} = Karas::Loader->load(
+        dbh => $self->dbh,
+        %args
+    );
+    return undef;
 }
 
 sub get_row_class {
-    my ($self, $table) = @_;
-    Carp::croak("Missing mandatory parameter: table") unless $table;
-    return $self->row_class_map->{$table} || $self->default_row_class;
+    my ($self, $table_name) = @_;
+    Carp::croak("Missing mandatory parameter: table_name") unless $table_name;
+    my $row_class = $self->row_class_map->{$table_name}
+        or Carp::croak("Unknown table: $table_name. $table_name is not registered to Karas.");
+    return $row_class;
 }
 
 # -------------------------------------------------------------------------
@@ -199,7 +192,10 @@ sub search_by_sql {
     my $sth = $self->dbh->prepare($sql);
     $sth->execute(@$binds);
     $table_name //= $self->guess_table_name($sql);
-    my $row_class = $table_name ? $self->get_row_class($table_name) : $self->default_row_class;
+    unless ($table_name) {
+        Carp::croak("Cannot guess table name from SQL. You need to pass the  table name: " . $sql);
+    }
+    my $row_class = $self->get_row_class($table_name);
     my @rows;
     while (my $row = $sth->fetchrow_hashref) {
         push @rows, $row_class->new($table_name, $row);
@@ -446,12 +442,6 @@ You can pass following arguments as hash:
 
 connect_info is an arguments for C<< DBI->connect >>.
 
-=item default_row_class(Optional)
-
-This is a default row class.
-
-Default value is : B<Karas::Row>.
-
 =item query_builder(Optional)
 
 This is a query builder. You need to pass the child class instance of SQL::Maker.
@@ -544,10 +534,6 @@ This is a bulk insert method. see L<SQL::Maker::Plugin::InsertMulti>.
 
 =over 4
 
-=item $db->clear_row_class_map_cache()
-
-Clear the row class map cache.
-
 =item $db->get_row_class($table_name);
 
 Clear row class from table name.
@@ -597,10 +583,6 @@ But if you use it as a parent class like following:
     parent MyDB;
     use parent qw/Karas/;
 
-Karas loads MyDB::Row::* automatically.
-
-Result of MyDB::Row is cached. You can clear cache by C<< Karas->clear_row_class_map_cache >>.
-
 =head1 FAQ
 
 =over 4
